#!/bin/bash

# echo commands
set -x
# exit on error
set -e

# save folder of script for location of the patches
patch_folder=$(readlink -f $(dirname $0))

sudo apt install git python haskell-stack flex bison libreadline-dev bc

# collect binaries in this folder
cd ~
mkdir -p ~/bin
echo "export PATH=\$PATH:\$HOME/bin:\$HOME/.local/bin" >>~/.bashrc

# install syfco
cd ~
git clone https://github.com/reactive-systems/syfco.git
cd syfco
stack setup
stack build
stack install
cd ~
rm -rf syfco

# install combine-aiger (use sparse checkout)
cd ~
mkdir syntcomp
cd syntcomp
git init
git remote add -f origin https://bitbucket.org/swenjacobs/syntcomp.git
git config core.sparsecheckout true
echo "tools/aiger-ltl-model-checker/" >> .git/info/sparse-checkout
git pull origin master
cd tools/aiger-ltl-model-checker
make
cp combine-aiger ~/bin
cd ~
rm -rf syntcomp

# install NuSMV
firefox http://nusmv.fbk.eu/NuSMV/download/getting_src-v2.html &
echo "Please download NuSMV from the displayed page"
echo "Afterwards, there should be a file 'NuSMV-2.6.0.tar.gz' in the 'Downloads' folder"
read -p "Then, press [Enter] to continue with the installation..."
cd ~
tar -xzf Downloads/NuSMV-2.6.0.tar.gz
patch -p0 <$patch_folder/nusmv_minisat.patch
patch -p0 <$patch_folder/nusmv_cudd.patch
cd NuSMV-2.6.0/NuSMV
mkdir build
cd build
cmake ..
make
cp bin/ltl2smv ~/bin
cd ~
rm -r NuSMV-2.6.0

# install nuXmv
firefox https://es-static.fbk.eu/tools/nuxmv/index.php?n=Download.Download &
echo "Please download nuXmv from the displayed page"
echo "Afterwards, there should be a file 'nuXmv-1.1.1-linux64.tar.gz' in the 'Downloads' folder"
read -p "Then, press [Enter] to continue with the installation..."
cd ~
tar -xzf Downloads/nuXmv-1.1.1-linux64.tar.gz nuXmv-1.1.1-Linux/bin/nuXmv
cp nuXmv-1.1.1-Linux/bin/nuXmv ~/bin
rm -r nuXmv-1.1.1-Linux

# install aiger tools
wget http://fmv.jku.at/aiger/aiger-1.9.4.tar.gz
tar -xzf aiger-1.9.4.tar.gz
cd aiger-1.9.4
./configure
make
cp smvtoaig ~/bin
cd ~
rm -r aiger-1.9.4 aiger-1.9.4.tar.gz
