#!/bin/bash

# exit on error
set -e
# echo commands
set -x
# break when pipe fails
set -o pipefail

# tool paths
STRIX=$1
SYFCO=syfco
SMVTOAIG=smvtoaig
COMBINEAIGER=combine-aiger
NUXMV=nuXmv
LTL2SMV=ltl2smv

# input file
TLSF_FILE=$2
BASE_FILE=$(basename ${2%.tlsf})

# temporary files
CONTROLLER_AIGER_FILE=/tmp/$BASE_FILE.aag
MONITOR_AIGER_FILE=/tmp/$BASE_FILE.monitor.aag
COMBINED_FILE=/tmp/$BASE_FILE.combined.aag
SMV_FILE=/tmp/$BASE_FILE.smv
RESULT_FILE=/tmp/$BASE_FILE.result
OUT_FILE=/tmp/$BASE_FILE.out

STRIX_OPTIONS='--validate-jni -e pq -c --auto'

# type of test
TEST=$3

# run the tool
LTL=$($SYFCO -f ltlxba -m fully $TLSF_FILE)
INS=$($SYFCO -f ltlxba --print-input-signals $TLSF_FILE)
OUTS=$($SYFCO -f ltlxba --print-output-signals $TLSF_FILE)
$STRIX -f "$LTL" --ins "$INS" --outs "$OUTS" -o $CONTROLLER_AIGER_FILE $STRIX_OPTIONS >$OUT_FILE

if [[ "$TEST" == "REALIZABLE" ]]; then
    # check if tool answers realizable
    grep -q "^REALIZABLE$" $OUT_FILE

    # build a monitor for the formula
    $SYFCO -f smv -m fully $TLSF_FILE >$SMV_FILE
    $SMVTOAIG -L $LTL2SMV $SMV_FILE -a >$MONITOR_AIGER_FILE
    $COMBINEAIGER $MONITOR_AIGER_FILE $CONTROLLER_AIGER_FILE >$COMBINED_FILE

    # model check solution from tool
    echo "read_aiger_model -i ${COMBINED_FILE}; encode_variables; build_boolean_model; check_ltlspec_klive; quit" | $NUXMV -int >$RESULT_FILE

    # check result
    grep -q 'specification .* is true' $RESULT_FILE
elif [[ "$TEST" == "UNREALIZABLE" ]]; then
    # check if tool answers unrealizable
    grep -q "^UNREALIZABLE$" $OUT_FILE
else
    # unknown test
    echo "Unknown test: $TEST"
    exit 2
fi
